/* 
 * Copyright (C) 2001 Igor Sysoev
 *
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 /

/*
  module works on FreeBSD only

  copy module to src/modules/extra/

  run ./configure
        ...
        --enable-module=src/modules/extra/mod_class.o
        ...

  add class to /etc/login.conf
  add to httpd.conf:
     LoginClass class_name
*/

/*
MODULE-DEFINITION-START
Name: class_module
ConfigStart
  LIBS="$LIBS -lutil"
  echo " + using -lutil for setclasscontext()"
ConfigEnd
MODULE-DEFINITION-END
*/

#include "httpd.h"
#include "http_config.h"
#include "http_log.h"
#include <login_cap.h>

#define MOD_CLASS_VER   "mod_class/1.0.0"

typedef struct {
    char *class;
} server_conf;

module class_module;

static void class_init(server_rec *s, pool *p)
{
    server_conf *conf = (server_conf *)
             ap_get_module_config(s->module_config, &class_module);

    /* setclassconext() returns zero even some rlimits can not be set
       due to denied permission, in this case it send message about it
       to syslogd only */

    if (conf->class)
        if (setclasscontext(conf->class, LOGIN_SETRESOURCES) == -1)
            ap_log_error(APLOG_MARK, APLOG_ERR, s,
                         "mod_class: setclasscontext() error");
}

static void *class_config(pool *p, server_rec *s)
{
    return (void *)ap_pcalloc(p, sizeof(server_conf));
}

static const char *set_class(cmd_parms *cmd, void *dummy, char *class)
{
    server_conf *conf = (server_conf *)
             ap_get_module_config(cmd->server->module_config, &class_module);

    conf->class = class;

    return NULL;
}

static const command_rec class_cmds[] =
{
    {"LoginClass", set_class, NULL, RSRC_CONF, TAKE1,
     "set login class"},

    {NULL}
};


module class_module =
{
    STANDARD_MODULE_STUFF,
    class_init,                       /* module initializer */
    NULL,                             /* per-directory config creator */
    NULL,                             /* dir config merger */
    class_config,                     /* server config creator */
    NULL,                             /* server config merger */
    class_cmds,                       /* command table */
    NULL,                             /* [9] list of handlers */
    NULL,                             /* [2] filename-to-URI translation */
    NULL,                             /* [5] check/validate user_id */
    NULL,                             /* [6] check user_id is valid *here* */
    NULL,                             /* [4] check access by host address */
    NULL,                             /* [7] MIME type checker/setter */
    NULL,                             /* [8] fixups */
    NULL,                             /* [10] logger */
    NULL,                             /* [3] header parser */
    NULL,                             /* process initializer */
    NULL,                             /* process exit/cleanup */   
    NULL,                             /* [1] post read_request handling */
}; 
